/*************************************************************************/
/*                                                                       */
/* FILE NAME                                      VERSION                */
/*                                                                       */
/*      iic.c                      KS32C5000, KS32C50100   : version 1.0 */
/*                                                                       */
/* COMPONENT                                                             */
/*                                                                       */
/*                                                                       */
/* DESCRIPTION                                                           */
/*                                                                       */
/*      I2C control function library for SNDS100 evaluation board.       */
/*                                                                       */
/* AUTHOR                                                                */
/*                                                                       */
/*                                                                       */
/* DATA STRUCTURES                                                       */
/*                                                                       */
/*                                                                       */
/* FUNCTIONS                                                             */
/*                                                                       */
/*                                                                       */
/* DEPENDENCIES                                                          */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      05-04-1999      Create IIC function libraries    */
/*************************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002,2003		*/
/*			*/

#include "evm50100.h"
#include "ks32c50.h"

////////////////////////////////////////////////////////////////////////////////
// uncomment IIC_DEBUG to debug interrupt mode read & write byte functions /////
////////////////////////////////////////////////////////////////////////////////
//#define IIC_DEBUG 

////////////////////////////////////////////////////////////////////////////////
// uncomment IIC_BYTE_DEBUG to debug pooling mode read & write byte function ///
////////////////////////////////////////////////////////////////////////////////
//#define IIC_BYTE_DEBUG 

// Serial EEPROM type //////////////////////////////////////////////////////////
#define IIC_EEPROM_AT24C01	1	// AT24C01 - 1K ( 128 x 8) ( 8 byte page write)
#define IIC_EEPROM_AT24C02	2	// AT24C02 - 2K ( 256 x 8) ( 8 byte page write)	
#define IIC_EEPROM_AT24C04	4	// AT24C04 - 4K ( 512 x 8) (16 byte page write)
#define IIC_EEPROM_AT24C08	8	// AT24C08 - 8K (1024 x 8) (16 byte page write)
#define IIC_EEPROM_AT24C16     16	// AT24C16 -16K (2048 x 8) (16 byte page write)

static U32 IIC_EEPROM_type    = 4; 	// AT24C04
static U32 IIC_clock_hz  = 100000;	// IIC Serial clock frequency 400KHz
static U8  IIC_EEPROM_addr = 0xa0;	// IIC device's slave address

// 
#define    IIC_buffer_size ((16 * 1024) / 8)  // IIC Test Data buffer size
static U8  IIC_buffer[IIC_buffer_size];	      // IIC Data buffer

// 
#define    IIC_mac_addr_start  0	// MAC address offset 
#define    IIC_mac_addr_size   6	// MAC address size
#define    IIC_test_addr       6	// IIC test start address 

#define    S_WRITE	    0x00	// Write IIC data for slave
#define    S_READ	    0x01	// Read IIC data for slave


//       +---+---+---+---+---+---+---+---+
// 1k/2K | 1 | 0 | 1 | 0 | A2| A1| A0|R/W| AT24C01A/AT24C02
//       +---+---+---+---+---+---+---+---+
//       +---+---+---+---+---+---+---+---+
// 4k    | 1 | 0 | 1 | 0 | A2| A1| P0|R/W| AT24C04
//       +---+---+---+---+---+---+---+---+
//       +---+---+---+---+---+---+---+---+
// 8k    | 1 | 0 | 1 | 0 | A2| P1| P0|R/W| AT24C08
//       +---+---+---+---+---+---+---+---+
//       +---+---+---+---+---+---+---+---+
//16k    | 1 | 0 | 1 | 0 | P2| P1| P0|R/W| AT24C16
//       +---+---+---+---+---+---+---+---+
// P0,P1,P2 - Page Address

////////////////////////////////////////////////////////////////////////////////
// return sizeof serial EEPROM in bytes ////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


static int IICEEPROMSize(void)
{
 switch(IIC_EEPROM_type)
  {
   case IIC_EEPROM_AT24C01 : return( 128); // AT24C01 - 1K ( 128 x 8) ( 8 byte page write)
   case IIC_EEPROM_AT24C02 : return( 256); // AT24C02 - 2K ( 256 x 8) ( 8 byte page write)
   case IIC_EEPROM_AT24C04 : return( 512); // AT24C04 - 4K ( 512 x 8) (16 byte page write)
   case IIC_EEPROM_AT24C08 : return(1024); // AT24C08 - 8K (1024 x 8) (16 byte page write)
   case IIC_EEPROM_AT24C16 : return(2048); // AT24C16 -16K (2048 x 8) (16 byte page write)
   default                 : return(0);
  }
}

////////////////////////////////////////////////////////////////////////////////
// return EEPROM name //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static char * IICEEPROMName(void)
{
 switch(IIC_EEPROM_type)
  {
   case IIC_EEPROM_AT24C01 : return("AT24C01"); // AT24C01 - 1K ( 128 x 8) ( 8 byte page write)
   case IIC_EEPROM_AT24C02 : return("AT24C02"); // AT24C02 - 2K ( 256 x 8) ( 8 byte page write)
   case IIC_EEPROM_AT24C04 : return("AT24C04"); // AT24C04 - 4K ( 512 x 8) (16 byte page write)
   case IIC_EEPROM_AT24C08 : return("AT24C08"); // AT24C08 - 8K (1024 x 8) (16 byte page write)
   case IIC_EEPROM_AT24C16 : return("AT24C16"); // AT24C16 -16K (2048 x 8) (16 byte page write)
   default                 :
    return(0);
  }
}

////////////////////////////////////////////////////////////////////////////////
// return device address ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


static U8 IICEEPROMAddr(void)
{ 
 U8 devaddr; 				// EEPROM address
 devaddr = (IIC_EEPROM_addr >> 1);	// bit 0 = R/W bit 
 switch(IIC_EEPROM_type)
  {
   case IIC_EEPROM_AT24C01 : devaddr &= 0x7; break; // A0,A1,A2
   case IIC_EEPROM_AT24C02 : devaddr &= 0x7; break; // A0,A1,A2
   case IIC_EEPROM_AT24C04 : devaddr &= 0x6; break; // A1,A2. A0 used as page number  
   case IIC_EEPROM_AT24C08 : devaddr &= 0x4; break; // A0,A1 used as page number 
   case IIC_EEPROM_AT24C16 : devaddr &= 0x0; break; // A0,A1,A2 used as a page number
   default                 : devaddr  = 0x0; break;  
  }

 return(devaddr);
}

////////////////////////////////////////////////////////////////////////////////
// IIC SETUP ROUTINE ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void IICSetup(void)
{
 U32 iicps0;				// prescale devider 0
 U32 iicps1;				// prescale devider 1	
 U32 clk0;				// result IIC clk for devider 0
 U32 clk1;				// result IIC clk for devider 1
 U32 delta0;				// clk 0 delta  
 U32 delta1;				// clk 1 delta
 U32 iicps;

// Reset IIC Controller ////////////////////////////////////////////////////////
 IICCON = IICCON_RESET;			// reset IIC controller

// clk = fMCLK / (16 * (iicps + 1) + 3);

 iicps0 = (fMCLK / IIC_clock_hz - 3) / 16 - 1;	// divider 0
 iicps1 =  iicps0 + 1;				// divider 1 

// 
 clk0 = fMCLK / (16 * (iicps0+1) + 3);	// clock 0
 clk1 = fMCLK / (16 * (iicps1+1) + 3);	// clock 1
 
 if(clk0 >= IIC_clock_hz)
  {
   delta0 = clk0 - IIC_clock_hz;
  }
 else
  {
   delta0 = IIC_clock_hz - clk0;	
  }

 if(clk1 >= IIC_clock_hz)
  {
   delta1 = clk1 - IIC_clock_hz;
  }
 else
  {
   delta1 = IIC_clock_hz - clk1;
  }

 if(delta0 <= delta1)
  {
   iicps = iicps0;
  }
 else
  {
   iicps = iicps1;
  }

//   
#if 0 
 Print("\niicps0=%d,iicps1=%d,clk0=%d,clk1=%d,delta0=%d,delta1=%d,iicps=%d\n",
  iicps0	,	
  iicps1	,
  clk0	,
  clk1	,
  delta0	,	
  delta1	,
  iicps  
 );
// 
#endif 

 IICPS = iicps;
}

////////////////////////////////////////////////////////////////////////////////
// Print IIC Bus Configuration /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void IICConfig(void)
{
 U32 iiccon;				// IIC Control Register 
 U32 iiclk;				// IIC Clock 		
 U32 iicps;				// IIC Prescaler Register 
 U32 iicbuf;				// IIC Shift Buffer Register
 U32 iiccnt;				// IIC Prescaler Counter Register
 U8 first=1;
 U8 it;

 while(1)
  {
   Print("\n\nIIC BUS Configuration");

////////////////////////////////////////////////////////////////////////////////
// IIC Control Register ////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

   iiccon = IICCON;			// IIC Control Register 
   Print("\nIICCON IIC Control Register =0x%08x", iiccon);

// [0] Buffer flag (BF) (1=Tx-empty,Rx-full) ///////////////////////////////////
   Print("\n0   [F]Buffer flag (BF).........%s",
    ((iiccon & IICCON_BF) != 0) ?
     "1(Tx empty,Rx full)" :
     "0(Tx not empty,Rx not full)"
   );

// [1] Interrupt enable (IEN) //////////////////////////////////////////////////
   Print("\n1   [I]Interrupt enable (IEN)...%s",
    ((iiccon & IICCON_IEN) != 0) ?
     "1(enabled)" : "0(disabled)"
   );

// [2] Last received bit (LRB) /////////////////////////////////////////////////
   Print("\n2      Last received bit (LRB)..%s", 
    ((iiccon & IICCON_LRB) != 0) ?
     "1(ACK not received)" : "0(ACK is received)"
   ); 

// [3]  Acknowledge enable (ACK) ///////////////////////////////////////////////
   Print("\n3   [A]Acknowledge enable (ACK).%s",
    ((iiccon & IICCON_ACK) != 0) ?
     "1(Generate ACK)" : "0"
   );
 
// [5:4] These bits control the generation of the Start, Stop,Repeat Start /////

   Print("\n5:4 [C]COND1, COND0.............");
   switch(iiccon &IICCON_COND)		// COND1, COND0	   	
    {
     case IICCON_COND_NULL	:	// 00 = no effect
      Print("00(no effect)");
      break;

     case IICCON_COND_START	:	// 01 = Start
      Print("01(Start)");
      break;
 
     case IICCON_COND_STOP	:	// 10 = Stop
      Print("10(Stop)");
      break;

     case IICCON_COND_RESTART	:	// 11 = Repeat Start
      Print("11(Repeat Start)");
      break;
//
     default	:
      break; 
    }

// [6]  Bus busy (BUSY) ////////////////////////////////////////////////////////

   Print("\n6      Bus busy (BUSY)..........%s",
    ((iiccon & IICCON_BUSY) != 0) ?
     "1(Bus busy)" : "0"
   );

   Print("\n7   [R]Reset....................%s", 
    ((iiccon & IICCON_RESET) != 0) ?
     "1" : "0"
   );
 
////////////////////////////////////////////////////////////////////////////////
// IIC Prescaler Register //////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

   iicps = IICPS;				// IIC Prescaler Register 
   Print("\n");
   Print("\nIICPS  Prescaler Register      =0x%08x", iicps);
   Print("\n15:0[P]Prescaler value..........0x%04x", iicps & 0xFFFF);
 
   iicps &= 0xFFFF;
   iiclk = fMCLK / (16 * (iicps + 1) + 3);
   Print("\n       IIC Clock................%d",iiclk);

////////////////////////////////////////////////////////////////////////////////
// IIC Shift Buffer Register ///////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

   if(first == 1) 
    {
     first  = 0;
     iicbuf = IICBUF;			// IIC Shift Buffer Register
    }
 
   Print("\n");
   Print("\nIICBUF Shift Buffer Register   =0x%08x", iicbuf);
   Print("\n7:0 [B]IIC Rx & Tx Data.........0x%02x", iicbuf & 0xFF);

////////////////////////////////////////////////////////////////////////////////
// IIC Prescaler Counter Register //////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

   iiccnt = IICCNT;			// IIC Prescaler Counter Register
   Print("\n");
   Print("\nIICCNT Prescaler Counter       =0x%08x", iiccnt);
   Print("\n15:0   IIC Counter value........0x%04x", iiccnt & 0xFFFF);

   Print("\n    [Q]Quit");
   Print("\nSelect Item");
   it = get_upper();
   switch(it)
    {
     case 'F' :
      {
// [0] Buffer flag (BF) (1=Tx-empty,Rx-full) ///////////////////////////////////
       Print("\n\nEnter Buffer flag (BF) [0/1]");
       it = get_upper();
       if(it == '0')
        {
         Print("\nBF set to 0(buffer not empty in Tx mode,not full in Rx mode)");
         IICCON &=~IICCON_BF;		// clear BF Flag 
        }
       else
       if(it == '1')
        {
         Print("\nBF set to 1(buffer empty in Tx mode,full in Rx mode).");
         IICCON |= IICCON_BF;		// set BF flag
        }
      }
      break;


     case 'I' :
// [1] Interrupt enable (IEN) //////////////////////////////////////////////////
      {
       Print("\nEnable Interrupt (IEN) [Y/N]");
       it = get_upper();
       if(it == 'Y')
        {
         IICCON |=  IICCON_IEN;		// interrupt enable 
        }
       else
       if(it == 'N')
        {
         IICCON &= ~IICCON_IEN;		// interrupt disable
        }
      }
      break; 
 
     case 'A' :
      {
// [3]  Acknowledge enable (ACK) ///////////////////////////////////////////////

       Print("\nAcknowledge enable (ACK) [Y/N]");
       it = get_upper();
       if(it == 'Y')
        {
         Print("\nGenerate an ACK signal at 9th SCL.");
         IICCON |=  IICCON_ACK;		// enable ACK
        }
       else
       if(it == 'N')
        {
         Print("\nDont Generate an ACK signal at 9th SCL.");
         IICCON &= ~IICCON_ACK;		// disable ACK
        } 
      }
      break;

// [5:4] These bits control the generation of the Start, Stop,Repeat Start /////
     case 'C' :
      {
       Print("\n[S]tart/s[T]op/[R]epeat start/[C]lear");
       it = get_upper();
// 
       iiccon = IICCON;			// read control register
// save control register bits BF,interrupt enable, ACK  	
       iiccon &= (IICCON_BF | IICCON_IEN | IICCON_ACK);
       if(it == 'S')
        {
         Print("\nStart.");		// start
         iiccon |= IICCON_COND_START;	// 01 = Start 
         IICCON  = iiccon;		// write IIC control register
        }
       else
       if(it == 'T')
        {
         Print("\nStop.");		// Stop 
         iiccon |= IICCON_COND_STOP;	// 10 = Stop
         IICCON  = iiccon;		// write IIC control register
        }
       else
       if(it == 'R')
        {
         Print("\nRepeat Start.");	// Start
         iiccon |= IICCON_COND_RESTART; // 11 = Repeat Start
         IICCON  = iiccon;		// write IIC control register
        }
       else
       if(it == 'C')
        {
         Print("\nClear (no effect).");
         iiccon |= IICCON_COND_NULL;	// 00 = no effect
         IICCON  = iiccon;		// write IIC control register
        }
      }
      break;
 
     case 'R' : 
      {
// [R] Reset ///////////////////////////////////////////////////////////////////
       Print("\nReset [Y/N]");
       it = get_upper(); 
       if(it == 'Y')
        {
         Print("\nReset IIC.");
         IICCON |= IICCON_RESET;	// reset IIC 
        }
      }
      break;

     case 'P' :
      {
//////////////////////////////////////////////////////////////////////////////// 
       iicps = IICPS & 0xFFFF;
       Print("\nEnter Prescaler value (current = 0x%04x) 0x", iicps);
       iicps = get_number(16,0);
       IICPS = (iicps & 0xFFFF);
      }
      break;

     case 'B' :
      {
//////////////////////////////////////////////////////////////////////////////// 
       Print("\nRead or Write IICBUF Shift Buffer Register [R/W]");
       it = get_upper();
       if(it == 'R')
        {
         if((IICCON & IICCON_BF) != 0)
          {
           Print("\nBF=1");
          }
         else
          {
           Print("\nBF=0");
          }
// 
         iicbuf = IICBUF;
         Print("\nIICBUF->0x%02x", iicbuf);

// 
         if((IICCON & IICCON_BF) != 0)
          {
           Print("\nBF=1");
          }
         else
          {
           Print("\nBF=0");
          }
        }
       else
       if(it == 'W')
        {         
         Print("\nEnter IICBUF Shift Buffer Register value 0x");

         iicbuf = get_number(16,2);

         if((IICCON & IICCON_BF) != 0)
          {
           Print("\nBF=1");
          }
         else
          {
           Print("\nBF=0");
          }

         Print("\nIICBUF<-0x%02x", iicbuf);
         IICBUF = iicbuf;

         if((IICCON & IICCON_BF) != 0)
          {
           Print("\nBF=1");
          }
         else
          {
           Print("\nBF=0");
          }
        }
       }
      break;

     case 'Q' : return;
     default  :
      break;
    }

   Print("\nPress any key to continue");
   it = get_upper();
   if(it == 'Q')
    {
     return;
    } 
  }
}


////////////////////////////////////////////////////////////////////////////////
// IIC Write Test Program //////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void IICWriteTest(void)
{
 char * name;				// Serial EEPROM name
 U32 addr;				// write address	
 U32 size;				// write size
 int pos;				// character position 
 int row;				// screen row 
 int col;				// screen collumn 
 U8 ch;					// console character 			

// 
 name = IICEEPROMName();		// read EEPROM name
 if(name == 0)
  {
   Print("\n\nIIC %s Write Test. Unknown serial EEPROM type.\n");
   return;
  }

 Print("\n\nIIC %s Write Test.", name);	// EEPROM type 

// 
 size = IICEEPROMSize();		// 
 Print("\nEnter Byte write address of IIC EEPROM (0-0x%x) 0x", size);
// 
 addr = get_number(16,0); 		// read EEPROM address from console 
 if(addr >= size)			// 
  {
// out from EEPROM size ////////////////////////////////////////////////////////
   Print("\nAddress [0x%x] not valid.\n", addr);
   return;				// 
  }

// 
 size = size - addr;			// count of writed bytes
 Print("\nInput Write Characters (up to %d bytes)", size);
 Print("\n<CR> to terminate input");

// read writed characters from  console ////////////////////////////////////////

 pos = 0; 				// position in test buffer
 while(1) 
  { 
   ch = get_byte();			// read byte from console  
   if(ch == KEY_CR)
    {
     size = pos;			// reset count of writed bytes 
     break;				// out off loop
    }

   IIC_buffer[pos++] = ch;		// write character to buffer 
   if(pos >= size)
    {					// 
     break;				// 
    }  
  }

// 
 Print("\n%d bytes received : ", size);
 if(size == 0)
  {					// cancel write test 				
   Print("\n");
   return;				// exit from EEPROM write test 
  } 


// print input from console characters /////////////////////////////////////////

 row = 0;				// current row 
 col = 0;				// curent collumn 
 pos = 0;				// current position in write buffer 
 while(pos < size) 
  {
   Print("%02x ", IIC_buffer[pos] );	// put current character  
   pos++;  				// to next next position in tx buffer
   col++; 				// next screen collumn 
   if(col >= 16)
    {
// to next screen row //////////////////////////////////////////////////////////
     Print("\n");
     col = 0;				// reset collumn 
     row++;				// row count 
     if(row >= 16)
      {
       while(kbd_hit())
        {				// clear input 
         ch = get_byte();		// from console 
        }
       Print("\nPress any key to continue ...");
       ch = get_byte();			// 
       row = 0;				// reset row 	
      }
    }
  }

// 
 pos = addr + size;  			// end + 1 writed address in EEPROM 

// 
 if(
    addr >= IIC_mac_addr_start + 	// start writed address > MAC end address
            IIC_mac_addr_size ||
     pos <=  IIC_mac_addr_start		// end writed adrress   < MAC start address
  )
  {
  }
 else 
  {
// Writed area averlap MAC address /////////////////////////////////////////////

   Print("\nMAC address stored in EEPROM from [0x%x] to [0x%x]",
     IIC_mac_addr_start	, 		// MAC address start 
     IIC_mac_addr_start + 		// MAC address end
     IIC_mac_addr_size  - 1
    );

   Print("\nOvewrite MAC address [Y/N]");
   ch = get_upper();
   if(ch == 'N')
    { 		
// exit from EEPROM write test /////////////////////////////////////////////////
     return;				
    } 
  }

// 
 Print("\nIIC Serial EEPROM %s write. Address=[0x%x]. Size=[0x%x].\n",
   name		, 			// EEPROM type 
   addr		,			// start EEPROM address
   size					// writed size
  );

// 
 IICWrite
  (
   IIC_EEPROM_addr	,		// device address
   (U32) addr		,		// EPPROM address
   IIC_buffer		,		// data buffer 
   size					// size
  );
}


////////////////////////////////////////////////////////////////////////////////
// IIC Read Test ///////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void IICReadTest(void)
{
 char * name;				// EEPROM name
 U32 addr;				// EEPROM address
 U32 size;				// read bytes count 
 U32 maxsize; 
 U32 pos;				// current position 
 U32 row;				// current screen row 
 U32 col;				// current screen col 
 U8 ch;

// 
 name = IICEEPROMName();		// read EEPROM name
 if(name == 0)
  {
   Print("\n\nIIC %s Read Test. Unknown serial EEPROM type.\n");
   return;
  }

//
 Print("\n\nIIC %s Read Test", name);

// read from console EEPROM address //////////////////////////////////////////// 

 size = IICEEPROMSize();
 Print("\nEnter Byte Read address of IIC EEPROM (up to [0x%x]) ", 
   size					// up limit off address 
  );

// 
 addr = get_number(16,0);		// read address from console 
 if(addr >= size)			// check input address
  {
   Print("\nByte address [0x%x] not valid\n", addr);
   return;
  }

// read from console byte count //////////////////////////////////////////////// 

 maxsize = size - addr;			// 
 Print("\nInput Read Data Size (up to [0x%x] bytes) ", maxsize);
 size = get_number(16,0);
 if(size > maxsize)
  {
   Print("\nNot correct data Size [0x%x]", size);
   return;
  }

// 
 Print("\nRead Data From IIC Serial EEPROM\n");
 IICRead
  (
   IIC_EEPROM_addr	,		// device address
   (U32) addr		,		// start address 
   IIC_buffer		, 		// data buffer 
   size					// size 
  );

// out read from EEPROM charactes from console /////////////////////////////////

 row = 0;				// current screen row 
 col = 0;				// current screen col
 pos = 0;
 while(pos < size) 
  {
   Print("%02x ", IIC_buffer[pos] );	// put current character  
   pos++;  				// next position in read buffer  
   col++; 				// next screen column 
   if(col >= 16)
    {
     Print("\n");
     col = 0;				// reset column 
     row++;				// next row
     if(row >= 16)
      {
       while(kbd_hit())
        {				// clear input 
         ch = get_byte();		// from console 
        }
       Print("\nPress any key to continue ...");
       ch = get_byte();		
       row = 0;				// reset row
      }
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
// IIC Read/Write Test Program by interrupt method /////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int IICWriteReadTest(int lcnt)
{
 U8  txdata[16];			// write data buffer 
 U8  rxdata[16];			// read  data buffer
 U32 size = 16;				// size of read & write buffers
 U32 pos;				// EEPROM start address
 U32 i;
 int rvl = 1;
 int lnum;

 Print("\n\nIIC Read/Write Test.");
 if(lcnt == 0)
  {
   Print("\nInput pass count ");
   lcnt = get_number(10, 3);
   if(lcnt == 0)
    {
     lcnt = 1;
    } 
  }

 for(lnum = 0; lnum < lcnt; ++lnum)
  {
// 
   if(lcnt > 1){ Print("\n%d.IIC Read/Write Test", lnum); }
   pos = IIC_test_addr; 		// current EEPROM address
   while(pos + size < IICEEPROMSize())	// in tested area
    {
// generate test buffer ////////////////////////////////////////////////////////
     for(i = 0; i < 16; ++i)
      {
       txdata[i] = (U8) PatternGen(pos + i);
      }

// Write IIC test pattern to Serial EEPROM /////////////////////////////////////
     IICWrite
      (
        IIC_EEPROM_addr	,		// device adress
        pos 		,		// EEPROM start adress
        txdata		,		// write data buffer
        size				// sizeof writed data
      );

/* Read IIC test pattern to Serial EEPROM */
     IICRead
      (
        IIC_EEPROM_addr	,		// device address
        pos		,		// EEPROM start adress
        rxdata		,		// read buffer 
        size				// sizeof read data
      );

     for(i = 0; i < 16; ++i)
      {
       if(txdata[i] != rxdata[i])
        {
         U8 ch;       
         IICReadByte(pos+i, &ch);
         Print("\nError Address [0x%x], Writed=[0x%02x], Read=[0x%02x], [0x%02x]",
           pos + i	, 		// address
           txdata[i] 	,		// writed value 
           rxdata[i]	,		// read value
           ch
         );
// 
         rvl = 0;
        }
      }
     pos += 1;				// to next EEPROM address
    }

   if(rvl == 0)
    {
     break;
    } 
  }
// 

 if(rvl == 1)
  {
   Print("\nOk.");
  }
 else
  {
   Print("\nFail.");
  } 

 return(rvl);
}

////////////////////////////////////////////////////////////////////////////////
// Select IIC EEPROM ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void IICEEPROMSelect(void)
{
 while(1)
  {
   char * tstr;
   U8 it;
// 
   switch(IIC_EEPROM_type)
    {
     case IIC_EEPROM_AT24C01: tstr = "AT24C01"; break;
     case IIC_EEPROM_AT24C02: tstr = "AT24C02"; break;	
     case IIC_EEPROM_AT24C04: tstr = "AT24C04"; break;	
     case IIC_EEPROM_AT24C08: tstr = "AT24C08"; break;
     case IIC_EEPROM_AT24C16: tstr = "AT24C16"; break;
     default: tstr = "Undefined"; break;
    }

   Print("\n\nSelect IIC EEPROM.");
   Print("\nCurrent [%s]", tstr);
   Print("\n[1] AT24C01 ( 1K, 128 x 8).");
   Print("\n[2] AT24C02 ( 2K, 256 x 8).");
   Print("\n[4] AT24C04 ( 4K, 512 x 8).");
   Print("\n[8] AT24C08 ( 8K,1024 x 8).");
   Print("\n[0] AT24C16 (16K,2048 x 8).");
   Print("\n[A] Select device address.");
   Print("\n[Q] Quit.");

   Print("\nSelect Item : ");
   it = get_upper();


   switch(it)
    {
     case '1' :
// AT24C01 - 1K ( 128 x 8) ( 8 byte page write) ////////////////////////////////
      Print("\nSelected AT24C01 (1K,128x8).");
      IIC_EEPROM_type = IIC_EEPROM_AT24C01;
      break;
      
     case '2' :
// AT24C02 - 2K ( 256 x 8) ( 8 byte page write)	////////////////////////////////
      Print("\nSelected AT24C02 (2K,256x8).");
      IIC_EEPROM_type = IIC_EEPROM_AT24C02;	
      break;

     case '4' :
// AT24C04 - 4K ( 512 x 8) (16 byte page write) ////////////////////////////////
      Print("\nSelected AT24C04 (4K,512x8).");
      IIC_EEPROM_type = IIC_EEPROM_AT24C04;	
      break;

     case '8' :
// AT24C08 - 8K (1024 x 8) (16 byte page write) ////////////////////////////////
      Print("\nSelected AT24C08 (8K,1024x8).");
      IIC_EEPROM_type = IIC_EEPROM_AT24C08;
      break;

     case '0' :
// AT24C16 -16K (2048 x 8) (16 byte page write) ////////////////////////////////
      Print("\nSelected AT24C16 (16K,2048x8).");
      IIC_EEPROM_type = IIC_EEPROM_AT24C16;	
      break;

     case 'A' :
      {
////////////////////////////////////////////////////////////////////////////////
// select slave device address /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\n\nSerial EEPROM type = %s.", IICEEPROMName());
       Print("\nEnter Device Address %d (0-7)", ((IIC_EEPROM_addr >> 1) & 0x7) );
       it = get_upper();
       if(it >= '0' && it <= '7')
        {
         it -= '0';			// device address
         switch(IIC_EEPROM_type)
          {
           case IIC_EEPROM_AT24C01 : // AT24C01
           case IIC_EEPROM_AT24C02 : // AT24C02
            IIC_EEPROM_addr = 0xa0 | (it << 1);
            break;

           case IIC_EEPROM_AT24C04 :
// AT24C04-4K //////////////////////////////////////////////////////////////////
            IIC_EEPROM_addr = 0xa0 | ((it & 0x6) << 1);
            if((it & 0x6) != it)
             {
              Print("\nIn AT24C04 A0 used as a page address.");
             }
            break;
            
           case IIC_EEPROM_AT24C08 : 
// AT24C08-8K //////////////////////////////////////////////////////////////////
            IIC_EEPROM_addr = 0xa0 | ((it & 0x4) << 1);
            if((it & 0x4) != it)
             {
              Print("\nIn AT24C08 A0,A1 used as a page address.");
             }
            break;

           case IIC_EEPROM_AT24C16 : 
// AT24C16 /////////////////////////////////////////////////////////////////////
            IIC_EEPROM_addr = 0xa0;
            if(it != 0)
             {
              Print("\nIn AT24C08 A0,A1,A2 used as a page address.");
             }
            break;

           default                 :
            break;
          }
//
         Print("\nDevice Address %d", ((IIC_EEPROM_addr >> 1) & 0x7));
        }
       else
        {
         Print("\nInvalid Device Address %c.\n",  it);
        } 
      }
      break;

     case 'Q' :				// exit from serial 
      return;				// EEPROM selection 
 
     default  :
      break;
    }

   Print("\nPress Any Key to Continue"); 
   it = get_upper();
   if(it == 'Q')
    {
     return;
    } 
  }
}

////////////////////////////////////////////////////////////////////////////////
// MAC Address Read from IIC EEPROM ////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int  MacAddrReadIIC(U8 * data)
{
 IICRead				// read IIC 
  ( 
   IIC_EEPROM_addr	,		// IIC device's slave address
   IIC_mac_addr_start   ,		// MAC address offset 
   data			, 		// data buffer 
   IIC_mac_addr_size			// data buffer size
  );

// 
 return(1);
}


////////////////////////////////////////////////////////////////////////////////
// MAC Address Write to IIC EEPROM /////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int MacAddrWriteIIC(U8 * data)
{
 IICWrite				// Write IIC 
  ( 
   IIC_EEPROM_addr	,		// IIC device's slave address
   IIC_mac_addr_start   ,		// MAC address offset 
   data			, 		// data buffer 
   IIC_mac_addr_size			// data buffer size
  );

// 
 return(1);
}


#ifdef IIC_BYTE_DEBUG
// Print BF flag ///////////////////////////////////////////////////////////////
/* The BF bit is set when : */
/* the buffer is empty in transmit mode, or */
/* the buffer is full in receive mode */
#define PrintIICBF()  Print("\nBF=%d",((IICCON & IICCON_BF) ? 1 : 0))
/* 0 = The most recent SDA is Low. (ACK is received) */
/* 1 = The most recent SDA is High. (ACK not received) */
#define PrintIICLRB() Print("\nACK %sRx",((IICCON & IICCON_LRB)==0) ? "":"not ") 
#define PrintIICMSG(msg) Print("\n%s",msg)
#define PrintIICBYTE(msg,ch) Print("\n%s0x%02x",msg,ch)
#else
#define PrintIICBF()
#define PrintIICLRB()
#define PrintIICMSG(msg)
#define PrintIICBYTE(msg,ch)
#endif					// IIC_BYTE_DEBUG

////////////////////////////////////////////////////////////////////////////////
// Wait Tx & Rx Complete ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define WaitIICBF()   while((IICCON & IICCON_BF) == 0) {;}

////////////////////////////////////////////////////////////////////////////////
// write byte to serial EEPROM in pooling mode /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int IICWriteByte(U32 addr, U8 ch)
{
// static int mcnt = 20000;
 U8 dev;				// device address

// disable interrupts <= byte write in pooling mode //////////////////////////// 
 Disable_Int(nIIC_INT);			// Disable IIC interrupt
 while((IICCON & IICCON_BUSY) != 0){;}

// Setup IICON register for transmit start /////////////////////////////////////

 PrintIICMSG("\nSTART BF");		// debug message START ACK BF
 IICCON = 
  IICCON_BF         | 			// tx buffer empty
  IICCON_COND_START; 			// Now, Start to transmit

////////////////////////////////////////////////////////////////////////////////
// Send Slave Address and Write command ////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 dev = IIC_EEPROM_addr;			// write page address to device addr
 switch(IIC_EEPROM_type)
  {
   case IIC_EEPROM_AT24C04 : dev = (dev & 0xFC) | ((addr >> 7) & 0x2); break;
   case IIC_EEPROM_AT24C08 : dev = (dev & 0xF8) | ((addr >> 7) & 0x6); break;
   case IIC_EEPROM_AT24C16 : dev = (dev & 0xF0) | ((addr >> 7) & 0xE); break;
   default  		   : dev = (dev & 0xFE); break;
  }

// 
 dev |= S_WRITE;			// write bit clear
 PrintIICBF();				// Print BF before IICBUF write
 PrintIICBYTE("<-", dev);		// Print writed device address
 IICBUF = dev;				// write device address
 PrintIICBF();				// Print BF after IICBUF write
 WaitIICBF();    			// wait  tx complete
 PrintIICLRB();				// Print ACK status

 if((IICCON & IICCON_LRB)){Print("\nByte Write ACK not received. Dev 0x%02x.",dev);}

////////////////////////////////////////////////////////////////////////////////
// write word address //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 PrintIICBF();				// Print BF before IICBUF write 
 PrintIICBYTE("<-", addr);		// print writed word address
 IICBUF = addr;				// write address 	
 PrintIICBF();				// Print BF after IICBUF write
 WaitIICBF();				// wait  tx complete
 PrintIICLRB();				// Print ACK Status

 if((IICCON & IICCON_LRB)){Print("\nByte Write ACK not received.Addr 0x%02x",addr);}

////////////////////////////////////////////////////////////////////////////////
// write byte //////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 PrintIICBYTE("<-", ch);		// print writed word
 IICBUF = ch;				// write byte
 PrintIICBF();				// Print BF after IICBUF write
 WaitIICBF();				// wait  tx complete
 PrintIICLRB();				// Print ACK status

 if((IICCON & IICCON_LRB)){Print("\nByte Write ACK not received. Data 0x%02x",ch);}

 PrintIICMSG("STOP");			// print STOP condition 
 IICCON = IICCON_COND_STOP;		// generate STOP condition

 {
  int wait;
  wait = (fMCLK / 1000 / 4 ) * 10;
  for(; wait > 0; --wait);  
 }
 return(1);
}


////////////////////////////////////////////////////////////////////////////////
// read byte from serial EEPROM in pooling mode ////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int IICReadByte(U32 addr, U8 * ch)
{
 U8 dev;				// device address
 if(ch == 0)
  {
   return(0);
  } 

// disable interrupts <= read byte in pooling mode /////////////////////////////

 Disable_Int(nIIC_INT);			// Disable IIC interrupt
 while((IICCON & IICCON_BUSY) != 0) {;}

////////////////////////////////////////////////////////////////////////////////
// generate start condition ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 PrintIICMSG("START BF");		// 

 IICCON = 
  IICCON_BF         | 			// Tx buffer empty (in transmit mode)
  IICCON_COND_START; 			// Now, Start to transmit

// Send Slave Address and Write command ////////////////////////////////////////
 dev = IIC_EEPROM_addr;
 switch(IIC_EEPROM_type)
  {
   case IIC_EEPROM_AT24C04 : dev = (dev & 0xFC) | ((addr >> 7) & 0x2); break;
   case IIC_EEPROM_AT24C08 : dev = (dev & 0xF8) | ((addr >> 7) & 0x6); break;
   case IIC_EEPROM_AT24C16 : dev = (dev & 0xF0) | ((addr >> 7) & 0xE); break;
   default 		   : dev = (dev & 0xFE); break;
  }


// send device address, page number, write bit ///////////////////////////////// 
 dev |= S_WRITE;			// 
 PrintIICBF();				// print BF before write
 PrintIICBYTE("<-", dev);		// print writed device address
 IICBUF = dev;				// Write to EEPROM device address
 PrintIICBF();				// print BF after write
 WaitIICBF();				// wait tx complete
 PrintIICLRB(); 			// print ACK status

 if((IICCON & IICCON_LRB)){Print("\nRead Byte ACK not received. Dev 0x%02x.",dev);}

////////////////////////////////////////////////////////////////////////////////
// send word address ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 PrintIICBYTE("<-", addr);		// print writed word address 
 IICBUF = addr;				// write address
 PrintIICBF();				// print BF after write
 WaitIICBF();				// wait tx complete
 PrintIICLRB(); 			// print ACK status

 if((IICCON & IICCON_LRB)){Print("\nRead Byte ACK not received.Addr 0x%02x.",addr);}

////////////////////////////////////////////////////////////////////////////////
// generate RESTART condition //////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 PrintIICMSG("RESTART");

//
 IICCON = IICCON_COND_RESTART; 		// Now, Restart to transmit

// send device addres, page bits, write/read bit /////////////////////////////// 
 dev = (dev & 0xFE) | S_READ;		// set read bit

////////////////////////////////////////////////////////////////////////////////
// generate start condition ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
 PrintIICMSG("START BF");
 IICCON = 
  IICCON_BF           | 		// Tx buffer empty (in tx mode)
  IICCON_COND_START;     		// Now, Start to transmit

 PrintIICBF();				// print BF before write
 PrintIICBYTE("<-", dev);		// print device address & read bit
 WaitIICBF();				// wait tx done 
 IICBUF = dev;				// Write to EEPROM device address
 PrintIICBF();				// print BF after write
 WaitIICBF();				// wait tx done 
 PrintIICLRB(); 			// print ACK status

 if((IICCON & IICCON_LRB)){Print("\nRead Byte ACK not received. Dev 0x%02x.",dev);}

////////////////////////////////////////////////////////////////////////////////
// read byte ///////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 *ch = IICBUF;				// read byte for clear BF
 PrintIICBYTE("->", *ch);		// print read byte
 WaitIICBF();				
 PrintIICBF();				// print BF after write
 PrintIICLRB(); 			// print ACK status

 *ch = IICBUF;			
 PrintIICBYTE("->", *ch);	
 WaitIICBF();	

////////////////////////////////////////////////////////////////////////////////
// generate stop condition /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 PrintIICMSG("IIC STOP");
 IICCON = IICCON_COND_STOP;
 return(1);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int IICByteTest(void)
{
 int lcnt;				// pass count  
 int lnum;				// pass num 
 int wskiped;				
 char * name; 				// serial EEPROM name
 U32 size;				// serial EEPROM size
 U32 pos;				// address
 int rvl;				// return code 
 U8 ch;					// 

// 
 rvl = 1;				// start return code 
 
// 
 size = IICEEPROMSize();		// calculate EEPROM size 	
 if(size > IIC_buffer_size)
  {
   size = IIC_buffer_size;
  }

 name = IICEEPROMName();
 Print("\n\nSerial EEPROM %s (%dbytes)byte write & read test.", 
   name	,				// EEPROM name
   size					// EEPROM size in bytes
  );

// test buffer start filling ///////////////////////////////////////////////////

 for(pos = 0; pos < size; ++pos)
  {					
   IIC_buffer[pos] = PatternGen(pos);		
  }  

// write to EEPROM /////////////////////////////////////////////////////////////
 Print("\nSkip write EEPROM [Y/N]");
 wskiped = get_upper();
 if(wskiped != 'Y')
  {
   U8 it;
   Print("\nRandom pattern    [Y/N]");
   it = get_upper();
   if(it != 'Y')
    {
// not random fill pattern /////////////////////////////////////////////////////
     for(pos = 0; pos < size; ++pos)
      {					
       IIC_buffer[pos] = pos;
      }  
    }
  }  

 lcnt = 10; 
 Print("\nInput pass count (%d) ", lcnt);
 lcnt = get_number(10, 0);
 if(lcnt == 0) { lcnt = 10; }
 
 for(lnum = 0; lnum < lcnt; ++lnum)
  {
   if(kbd_hit())
    {
     ch = get_byte(); 
     if(ch == 'Q' || ch == 'q')
      {
       break;
      } 
    }


   if(wskiped != 'Y')
    {
     Print("\n%d Write serial EEPROM...", lnum);
     for(pos = 0; pos < size; ++pos)
      {
       ch = IIC_buffer[pos];		// read character 
       IICWriteByte(pos,ch);		// write to EEPROM 
      }
    }

// read from EEPROM ////////////////////////////////////////////////////////////

   Print("\n%d Read serial EEPROM...", lnum);
   for(pos = 0; pos < size; ++pos)
    {
     IICReadByte(pos,& ch);

     if(ch != IIC_buffer[pos])
      {
       Print("\nError, address=[0x%x], write=[0x%02x], read=[0x%02x]",
        pos	      ,
        IIC_buffer[pos],       
        ch
       );

// 
       rvl = 0;
       if(kbd_hit())
        {
         get_byte(); 
         break;
        }
      }
    }

   if(rvl == 0)
    {
     break;
    } 
  }

 if(rvl == 1)
  {
   Print("\nOk.\n");
  }
 else
  {
   Print("\nFail.\n");
  }

// 
 return(rvl);
}


/* Byte Slave transmitter and receive status flags */

#define  IIC_DEV_ADDR      ((U32)0x01)  // device address
#define  IIC_BYTE_ADDR_MSB ((U32)0x02)  // byte msb addr
#define  IIC_BYTE_ADDR_LSB ((U32)0x04)  // byte lsb addr
#define  IIC_PAGE_TX_DONE  ((U32)0x08)  // Page data transmit done
#define  IIC_BYTE_RX_DONE  ((U32)0x10)  // byte data receive done
#define  IIC_REPEAT_START  ((U32)0x20)  // flag for repeat start
#define  IIC_MULTI_RECV    ((U32)0x40)  // recv multiple data
#define  IIC_NO_MORE_RECV  ((U32)0x80)  // no more recv from slave

/* Data structure for multiple byte slave transmitter format */

typedef struct 
{
 volatile U32 flag;
 volatile U32 count;          	// Page buffer byte counter
 volatile U32 size;	        // Page buffer byte counter
 volatile U32 noack;		// no ACK flag 
 volatile U8  dev;              // Slave address
 volatile U8  msb; 	        // Byte MSB address
 volatile U8  lsb; 	        // Byte LSB address
 volatile U8  buffer[16]; 	// IIC Page data buffer
} 
 IIC_DATA_TX_FORMAT;


IIC_DATA_TX_FORMAT  IIC_TX;

/* Data structure for multiple byte slave receiver format */

typedef struct 
{
 volatile U32 flag;
 volatile U32 count;             
 volatile U32 size;             
 volatile U8  dev;
 volatile U8  msb;
 volatile U8  lsb;
 volatile U32 noack;		// no ACK flag 
 volatile U8  * buffer;
} 
 IIC_DATA_RX_FORMAT;


IIC_DATA_RX_FORMAT  IIC_RX;


#ifdef IIC_DEBUG
// 
#define IIC_DEBUG_BUF_SIZE 128		// debug buffer size 
U16 IIC_DEBUG_BUF[IIC_DEBUG_BUF_SIZE];	// debig buffer 
U32 IIC_DEBUG_CNT = 0;			// debug cnt 

#define IIC_DEBUG_W_CON   (U16)(0x0100)	// write controll register
#define IIC_DEBUG_R_CON   (U16)(0x0200) // read  control register
#define IIC_DEBUG_W_BUF   (U16)(0x0400) // write buffer register 
#define IIC_DEBUG_R_BUF   (U16)(0x0800) // read buffer register 
#define IIC_DEBUG_INT     (U16)(0x4000) // int start
#define IIC_DEBUG_UNKNOWN (U16)(0x8000) // unknown interrupt 

// put debug message to buffer /////////////////////////////////////////////////

void IIC_DEBUG_PUT(U16 typ, U8 val)
{
 if(IIC_DEBUG_CNT >= IIC_DEBUG_BUF_SIZE)
  {					// debug buffer full
   return;
  }

 IIC_DEBUG_BUF[IIC_DEBUG_CNT++] = typ | val;
}

// print debug buffer //////////////////////////////////////////////////////////

void IIC_DEBUG_DUMP(void)
{
 U32 pos;				// position in debug buffer 
 for(pos = 0; pos < IIC_DEBUG_CNT; ++pos)
  {
   U16 val;				// debug message 
   val = IIC_DEBUG_BUF[pos]; 		// read debug message  

   if((val & IIC_DEBUG_INT) != 0)
    {
     Print("\nINT");
    }
   else
   if(
        (val & IIC_DEBUG_W_CON) != 0 ||	// write control register
        (val & IIC_DEBUG_R_CON) != 0	// read control register 
    )
    {
     if((val & IIC_DEBUG_W_CON) != 0)
      {
// write control register header ///////////////////////////////////////////////
       Print("\nW IICCON ");
      }
     else
      {
// read control register header ////////////////////////////////////////////////
       Print("\nR IICCON ");
      }

// condition code //////////////////////////////////////////////////////////////
     switch(val & IICCON_COND)
      {
       case IICCON_COND_NULL    : break;
       case IICCON_COND_START   : Print("START "); break;   // start condition 
       case IICCON_COND_STOP    : Print("STOP "); break;    // stop condition 
       case IICCON_COND_RESTART : Print("RESTART "); break; // restart condition 
      }

// 
     if((val & IICCON_BF   ) != 0) Print("BF ");	// 
     if((val & IICCON_IEN  ) != 0) Print("IEN ");	// 
     if((val & IICCON_LRB  ) != 0) Print("LRB ");	// 
     if((val & IICCON_ACK  ) != 0) Print("ACK ");	// 
     if((val & IICCON_BUSY ) != 0) Print("BUSY ");	// 
     if((val & IICCON_RESET) != 0) Print("RESET ");	// 
    }
   else
   if((val & IIC_DEBUG_W_BUF) != 0)
    {
// write buffer register ///////////////////////////////////////////////////////
     Print("\nW IICBUF %02x", val & 0xFF);	// 
    }
   else
   if((val & IIC_DEBUG_R_BUF) != 0)
    {
// read buffer register ////////////////////////////////////////////////////////
     Print("\nR IICBUF %02x", val & 0xFF);	// 
    } 
   else
    {
     Print("\nUNKNOWN");
    }
  }

 IIC_DEBUG_CNT = 0;
}

// write controll register /////////////////////////////////////////////////////
#define IIC_W_CON(val) {IIC_DEBUG_PUT(IIC_DEBUG_W_CON, val); IICCON = val;} 
// read  control register //////////////////////////////////////////////////////
#define IIC_R_CON(val) {val = IICCON; IIC_DEBUG_PUT(IIC_DEBUG_R_CON, val);}
// write buffer register ///////////////////////////////////////////////////////
#define IIC_W_BUF(val) {IIC_DEBUG_PUT(IIC_DEBUG_W_BUF, val); IICBUF = val;} 
// read buffer register ////////////////////////////////////////////////////////
#define IIC_R_BUF(val) {val = IICBUF; IIC_DEBUG_PUT(IIC_DEBUG_R_BUF, val);} 
// int start ///////////////////////////////////////////////////////////////////
#define IIC_S_INT()     IIC_DEBUG_PUT(IIC_DEBUG_INT,0);
// unknown interrupt /////////////////////////////////////////////////////////// 
#define IIC_UNKNOWN()   IIC_DEBUG_PUT(IIC_DEBUG_UNKNOWN,0);

#else
#define IIC_DEBUG_DUMP()
#define IIC_W_CON(val) 		IICCON = val 
#define IIC_R_CON(val)          val = IICCON
#define IIC_W_BUF(val) 		IICBUF = val 
#define IIC_R_BUF(val)          val = IICBUF
#define IIC_S_INT()    
#define IIC_UNKNOWN()
#endif 							// IIC_DEBUG


////////////////////////////////////////////////////////////////////////////////
// LIBRARY FUNCTIONS FOR IIC READ & WRITE //////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


static void IICWriteIsr(void);

// write page, return count of writed bytes  

static int IICWritePage(U32 type, U8 dev, U32 addr, U8 * data, U32 len)
{
 int pos; 

 while((IICCON & IICCON_BUSY) != 0){;} 	// wait IIC busy
// 
 switch(type)
  {
   case IIC_EEPROM_AT24C01 : 		// AT24C01
   case IIC_EEPROM_AT24C02 :		// AT24C02
    pos = 8 - (addr & 0x7);		//
    break;

   case IIC_EEPROM_AT24C04 : 		// AT24C04
   case IIC_EEPROM_AT24C08 :		// AT24C08
   case IIC_EEPROM_AT24C16 :		// AT24C16
    pos = 16 - (addr & 0xF);
    break;
   
   default  		   :
    return(0);
  }

 if(pos < len)
  {
   len = pos;
  } 

//
 Disable_Int(nGLOBAL_INT);		// 
 IIC_TX.flag  = 0x0;			// clear flag 
 IIC_TX.count = 0x0;			// current tx byte
 IIC_TX.msb   = 0;			// 8 bits of address
 IIC_TX.lsb   = addr & 0xff;		// 8 bits of address
 IIC_TX.size  = len;		        // Page buffer byte counter
 IIC_TX.noack = 0;			// no ACK flag 

 for(pos = 0; pos < len; ++pos)
  {
   IIC_TX.buffer[pos] = *data++;	// 
  }     

 switch(type)
  {
   case IIC_EEPROM_AT24C04 : dev = (dev & 0xFC) | ((addr >> 7) & 0x2); break;
   case IIC_EEPROM_AT24C08 : dev = (dev & 0xF8) | ((addr >> 7) & 0x6); break;
   case IIC_EEPROM_AT24C16 : dev = (dev & 0xF0) | ((addr >> 7) & 0xE); break;
   default                 : dev = (dev & 0xFE);   break;
  }

 dev |= S_WRITE;			// slave device address & page address
 IIC_TX.dev = dev;			// Write to EEPROM 	

 Enable_Int(nGLOBAL_INT);		// 
// Setup IICON register for transmit start /////////////////////////////////////
 IIC_W_CON(IICCON_BF|IICCON_COND_START|IICCON_IEN);

// Send Slave Address and Write command ////////////////////////////////////////

 while((IIC_TX.flag & IIC_PAGE_TX_DONE) == 0 && IIC_TX.noack == 0) {;}
 
 if(IIC_TX.noack != 0)
  {
   Print("\nPage write error. %d\nReset IIC", IIC_TX.noack);
   IICCON = IICCON_RESET;
   return(0);
  }

 {
  int wait;
  wait = (fMCLK / 1000 / 4 ) * 10;
  for(; wait > 0; --wait);  
 }
//
 return(len);
}


//
// dev  = device address
// addr = address 
// data = data buffer
// len  = data length 
// 

int IICWrite(U8 dev, U32 addr, U8 * data, U32 len)
{
 U32 size;
 int cnt;
// 
 switch(IIC_EEPROM_type)
  {
   case IIC_EEPROM_AT24C01 : size =  128; break;
   case IIC_EEPROM_AT24C02 : size =  256; break;
   case IIC_EEPROM_AT24C04 : size =  512; break;
   case IIC_EEPROM_AT24C08 : size = 1024; break;
   case IIC_EEPROM_AT24C16 : size = 2048; break;
  }

 if(data == 0 || len == 0 || addr + len > size)
  {
   return(0);
  } 

 SysSetInterrupt(nIIC_INT,IICWriteIsr); // Setup IIC Tx interrupt
 Enable_Int(nIIC_INT);			// Enable IIC interrupt 	

 while(len > 0)
  {
   cnt = IICWritePage(IIC_EEPROM_type, dev, addr, data, len);

   if(cnt == 0)
    {
     Print("\nIICWrite error");
     Disable_Int(nIIC_INT);			
     return(0);
    }

   len  -= cnt;   			// remain write bytes
   addr += cnt;				// move address 
   data += cnt;   			// move data pointer
  }

 Disable_Int(nIIC_INT);			
 return(1);
}


////////////////////////////////////////////////////////////////////////////////
// IIC INTERRUPT SERVICE ROUTINES //////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void IICWriteIsr(void)
{
 U32 iiccon;
 U8 ch;

 IIC_S_INT();

 if((IIC_TX.flag & IIC_DEV_ADDR) == 0) 
  {
   IIC_R_CON(iiccon);			// read control reg
   IIC_W_BUF(IIC_TX.dev);		// write dev address to buffer reg
   IIC_TX.flag |= IIC_DEV_ADDR;		// set write dev address done flag
  }
 else
 if((IIC_TX.flag & IIC_BYTE_ADDR_LSB) == 0) 
  {
// send write address //////////////////////////////////////////////////////////
   IIC_R_CON(iiccon);			// read control register
   if(iiccon & IICCON_LRB) 
    {					// not rx device address ack 
     IIC_TX.noack |= IIC_DEV_ADDR;
    } 
   IIC_W_BUF(IIC_TX.lsb);		// write address
   IIC_TX.flag |= IIC_BYTE_ADDR_LSB; 	// set write address done flag
  }
 else 
 if(IIC_TX.count < IIC_TX.size) 
  {
   IIC_R_CON(iiccon);			// read control register 
   if(iiccon & IICCON_LRB) 
    {					// no ack
     if(IIC_TX.count == 0)
      {
       IIC_TX.noack |= IIC_BYTE_ADDR_LSB;
      } 
     else
      {
       IIC_TX.noack |= IIC_PAGE_TX_DONE;
      } 
    } 

// send next character /////////////////////////////////////////////////////////
   ch = IIC_TX.buffer[IIC_TX.count++];	// 
   IIC_W_BUF(ch);			// write byte to buffer reg
  }
 else 
 if((IIC_TX.flag & IIC_PAGE_TX_DONE) == 0)
  { 
   IIC_R_CON(iiccon);
   if(IICCON & IICCON_LRB) 
    {
     IIC_TX.noack |= IIC_PAGE_TX_DONE;
    } 

// generate stop condition /////////////////////////////////////////////////////
/* STOP IIC Controller */
   IIC_W_CON(IICCON_COND_STOP);		// write control reg
   IIC_R_BUF(ch);   
/* byte data or page data transmit done */
   IIC_TX.flag |= IIC_PAGE_TX_DONE;
  }
 else
  {
   IIC_R_CON(iiccon);
   IIC_UNKNOWN();
  }
}

////////////////////////////////////////////////////////////////////////////////
// Read From Serial EEPROM /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void IICReadIsr(void);

void IICRead(U8 dev,U32 addr, U8 * data, U32 len)
{
 Disable_Int(nGLOBAL_INT);		// disable global Int
 SysSetInterrupt(nIIC_INT,IICReadIsr) ; // Setup IIC Rx interrupt
 Enable_Int(nIIC_INT);			// Enable IIC interrupt  

// Wait while the iic bus is busy //////////////////////////////////////////////
 while((IICCON & IICCON_BUSY) != 0) {;}

// 
 switch(IIC_EEPROM_type)
  {
   case IIC_EEPROM_AT24C04 : dev = (dev & 0xFC) | ((addr >> 7) & 0x2); break;
   case IIC_EEPROM_AT24C08 : dev = (dev & 0xF8) | ((addr >> 7) & 0x6); break;
   case IIC_EEPROM_AT24C16 : dev = (dev & 0xF0) | ((addr >> 7) & 0xE); break;
   default  		   : dev = (dev & 0xFE); break;
  }
 dev |= S_WRITE;			// 

// 
 IIC_RX.buffer = data;			// data buffer pointer 
 IIC_RX.flag  = 0x0;			// interrupt flag 
 IIC_RX.count = 0x0;			// read byte count
 IIC_RX.size  = len;			// read size
 IIC_RX.dev   = dev;			// device address 
 IIC_RX.msb   = (U8)((addr>>8) & 0xff);
 IIC_RX.lsb   = (U8) (addr & 0xff);	// low byte of address
 IIC_RX.noack = 0;

// Start, interrupt enable
 Enable_Int(nGLOBAL_INT);		// Enable global interrupt 
 IIC_W_CON(IICCON_BF | IICCON_COND_START | IICCON_IEN);
// 
 while((IIC_RX.flag & IIC_BYTE_RX_DONE) == 0 && IIC_RX.noack == 0)
  {
// wait rx done or no ack condition 
  }

 if(IIC_RX.noack != 0)
  {
   Print("\nIIC read error. No ack %d\nReset IIC", IIC_RX.noack);
   IICCON = IICCON_RESET;
  }

 Disable_Int(nIIC_INT);			// Enable IIC interrupt  
 IIC_DEBUG_DUMP();
}


////////////////////////////////////////////////////////////////////////////////
// IIC read ISR ////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void IICReadIsr(void)
{
 U32 iiccon;				// 
 U8 ch;					// 
 IIC_S_INT();

 if((IIC_RX.flag & IIC_DEV_ADDR) == 0) 
  {
   IIC_W_BUF(IIC_RX.dev);		// write dev addr 
   IIC_RX.flag |= IIC_DEV_ADDR;		// set write device address done flag  
  }
 else
 if((IIC_RX.flag & IIC_BYTE_ADDR_LSB) == 0) 
  {
////////////////////////////////////////////////////////////////////////////////
// send low byte of word address ///////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
   IIC_R_CON(iiccon); 			// read control reg
   if(iiccon & IICCON_LRB) 
    {
     IIC_RX.noack |= IIC_DEV_ADDR;
    }

   IIC_W_BUF(IIC_RX.lsb);		// write low byte of address word
   IIC_RX.flag |= IIC_BYTE_ADDR_LSB; 	// send lsb byte addr
  }
 else 
 if((IIC_RX.flag & IIC_REPEAT_START) == 0) 
  {
////////////////////////////////////////////////////////////////////////////////
// Generate Repeat Start ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
   IIC_R_CON(iiccon);
   if(iiccon & IICCON_LRB) 
    {
     IIC_RX.noack |= IIC_BYTE_ADDR_LSB;
    }

   IIC_W_CON(IICCON_COND_RESTART|IICCON_IEN);

// Generate Start //////////////////////////////////////////////////////////////
// BF, Start Condition, ACK enable, interrupt enable
   IIC_W_CON(IICCON_BF|IICCON_COND_START|IICCON_ACK|IICCON_IEN);

// Select device ///////////////////////////////////////////////////////////////
// device address// read bit 
   IIC_W_BUF((IIC_RX.dev | S_READ));
   IIC_RX.flag |= IIC_REPEAT_START;
  }
 else 
 if((IIC_RX.flag & IIC_MULTI_RECV) == 0) 
  {
/* Receive multiple data */
   IIC_R_CON(iiccon);
   if(IICCON & IICCON_BF)
    {
     IIC_RX.flag |= IIC_MULTI_RECV;
    }
    
   IIC_W_CON(IICCON_ACK|IICCON_IEN);
   IIC_R_BUF(ch);
  }
 else 
 if(IIC_RX.count < IIC_RX.size) 
  {
////////////////////////////////////////////////////////////////////////////////
// read received byte //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
   IIC_R_CON(iiccon);
   if(iiccon & IICCON_LRB) 
    {
     IIC_RX.noack |= IIC_MULTI_RECV;
    }

   IIC_R_BUF(ch);
   *(IIC_RX.buffer)++ = ch;		// write rx byte to buffer
     IIC_RX.count++;			// received bytes
  }
 else 
 if((IIC_RX.flag & IIC_NO_MORE_RECV) == 0) 
  { 
/* Now,no more received data is required from slave */
   IIC_W_CON(IICCON_NOACK | IICCON_IEN);
   IIC_RX.flag |= IIC_NO_MORE_RECV;
  }
 else 
 if((IIC_RX.flag & IIC_BYTE_RX_DONE) == 0)
  { 
/* Receive last data and STOP */
   IIC_R_BUF(ch);
   IIC_W_CON(IICCON_COND_STOP);
   IIC_R_BUF(ch);
/* byte data receive done */
   IIC_RX.flag |= IIC_BYTE_RX_DONE;
  }
 else
  {
   IIC_UNKNOWN();
  }
}

////////////////////////////////////////////////////////////////////////////////
// read from console address & byte value and write byte to serial EEPROM //////
////////////////////////////////////////////////////////////////////////////////

void IICWriteByteTest(void)
{
 U32 size;				// EEPROM size
 U32 addr;				// EEPROM address
 U8 ch;					// writed character

// 
 size = IICEEPROMSize();
 Print("\nEnter Byte Write address of IIC EEPROM (up to [0x%x]) ", 
   size					// up limit off address 
  );

// 
 addr = get_number(16,0);			// read address from console 
 if(addr >= size)			// check input address
  {
   Print("\nByte address [0x%x] not valid\n", addr);
   return;
  }

 while(kbd_hit())
  {					// clear input
   ch = get_byte();			// from console
  } 

// 
 Print("\nEnter writed byte 0x");
 ch = get_number(16,0); 			// read byte from console

 IICWriteByte(addr, ch);
}


////////////////////////////////////////////////////////////////////////////////
// read address from console and read byte from EEPROM /////////////////////////
////////////////////////////////////////////////////////////////////////////////

void IICReadByteTest(void)
{
 U32 size;				// serial EEPROM size
 U32 addr;				// addres
 int cnt;
 U8 ch;					// read character 

// 
 size = IICEEPROMSize();
 Print("\nEnter Byte Read address of IIC EEPROM (up to [0x%x]) ", 
   size					// up limit off address 
  );

// 
 addr = get_number(16,0);		// read address from console 
 if(addr >= size)			// check input address
  {
   Print("\nByte address [0x%x] not valid\n", addr);
   return;
  }

 Print("\nEnter count of bytes (up to %d) ", size - addr);
 cnt = get_number(10, 3);
 if(cnt > size - addr)
 {cnt = size - addr;} 
 if(cnt < 1) {cnt = 1;}

 Print("\n"); 
 while(cnt-- > 0)
  {
   if(IICReadByte(addr, & ch) != 0)
    {
     Print("0x%02x ", ch); 
    }
   addr++; 
  }
}


////////////////////////////////////////////////////////////////////////////////
// IIC Test using IIC 64Kbit Serial EEPROM /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void IICTest(void) 
{
 IICSetup();				// Initialize IIC control block
 Enable_Int(nGLOBAL_INT); 		// enable global interrupt mask

 while(1)
  {
   U8 it;

// print EEPROM name, device address
   Print("\n\nIIC %s Test. Device Address %d", 
    IICEEPROMName(), IICEEPROMAddr());
//
   Print("\n[W] IIC Page Write Test.");
   Print("\n[R] IIC Sequential Read Test.");
   Print("\n[L] IIC Read/Write Test.");
   Print("\n[C] IIC Configuration View.");
   Print("\n[S] IIC EEPROM type & address Select.");
   Print("\n[B] IIC Byte Write.");
   Print("\n[D] IIC Byte Read.");
   Print("\n[T] IIC Byte Write & Read Test.");
   Print("\n[Q] Quit IIC Test");

   Print("\nSelect Test Item : ");
   it = get_upper();

   switch(it)
    {
     case 'W' : IICWriteTest(); break;	   // EEPROM write 
     case 'R' : IICReadTest(); break;	   // EEPROM read 
     case 'L' : IICWriteReadTest(0); break; // EEPROM write & read test 
     case 'C' : IICConfig(); break;	   // IIC configuration 
     case 'S' : IICEEPROMSelect(); break;  // Select IIC EEPROM 	
     case 'B' : IICWriteByteTest(); break;
     case 'D' : IICReadByteTest(); break;
     case 'T' : IICByteTest(); break;	   // IIC Byte Write & Read Test

     case 'Q' : return;
     default  : break;
    }

// 
   Print("\nPress Any Key to Continue IIC Test") ; 
   it = get_upper();
   if(it == 'Q')
    {
     return;
    } 
  } 
}
