/*************************************************************************/
/*                                                                       */
/* FILE NAME                                      VERSION                */
/*                                                                       */
/*      timer.c                    KS32C5000, KS32C50100   : version 1.0 */
/*                                                                       */
/* COMPONENT                                                             */
/*                                                                       */
/*                                                                       */
/* DESCRIPTION                                                           */
/*                                                                       */
/*                                                                       */
/* AUTHOR                                                                */
/*                                                                       */
/*      Young Sun KIM, Samsung Electronics, Inc.                         */
/*                                                                       */
/* DATA STRUCTURES                                                       */
/*                                                                       */
/*                                                                       */
/* FUNCTIONS                                                             */
/*                                                                       */
/*   Timer basic library functions:                                      */
/*                                                                       */
/*    TimerInit      Initialize the timer with 1 sec unit                */
/*    tmReset        Clear timer counter and interrupt pending,          */
/*                   Restart timer                                       */
/*    tmCntr         Change the timer mode,periode,interrupt             */
/*    TimeStopCheck  Timer start & stop check in polling method          */
/*    PrtSysTime     System time will be displayed                       */
/*    GetSysTime     get system time display string                      */
/*    tm0isr         timer0 interrupt service routine                    */
/*    tm1isr         timer1 interrupt service routine                    */
/*                                                                       */
/* DEPENDENCIES                                                          */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      09-15-1998      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002			*/
/*			*/

#include "ks32c50.h"
#include "evm50100.h"

// uncomment to enable TOUTn pins 
//#define TIMER_TOUT

volatile unsigned int  clk_tick0 = 0;  		/* time tick for timer 0 */
volatile unsigned int  clk_tick1 = 0;  		/* time tick for timer 0 */
volatile unsigned int  WatchDogStatus = 0;  

/* Timer0,1 can be used for system real time clock */
TIME tm0 = 
 {
  0	,				/* millisec */
  0	,				/* sec */
  0	,				/* min */
  0	,				/* hour */
  0	,				/* month day */
  0	,				/* month */	
  0					/* year */
 };

TIME tm1 = 
 {
  0	,				/* millisec */
  0	,				/* sec */
  0	,				/* min */
  0	,				/* hour */
  0	,				/* month day */
  0	,				/* month */	
  0					/* year */
 };


void TimerStart(int t)
{
 if(t == 1)
  {
   TMOD |= TMOD1_EN;
  }
 else
 if(t == 0)
  {
   TMOD |= TMOD0_EN;
  }
} 

void TimerStop(int t)
{
 if(t == 1)
  {
   TMOD &= ~TMOD1_EN;
  }
 else
 if(t == 0)
  {
   TMOD &= ~TMOD0_EN;
  }
}  


/*************************************************************************/
/*                                                                       */
/* NAME : tmConfig()                                                     */
/*                                                                       */
/* FUNCTIONS : Monitoring the timer related system registers.            */
/*             and start & stop timer control.                           */
/*                                                                       */
/* EXAMPLES :                                                            */
/*                                                                       */
/* VARIABLES USED                                                        */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      06-07-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/


static void TimerConfigView(void)
{
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
 Print("\nTIMER 0 Status");
// TMOD0_EN // Timer 0 enable //////////////////////////////////////////////////
 Print("\nTimer 0 enable..............%s", 
  (TMOD & TMOD0_EN) 	?   
  "1(enabled."		:
  "0(disabled)"
  );

// TMOD0_TOGGLE // interval mode ///////////////////////////////////////////////
 Print("\nTimer 0 Toggle mode.........%s", 
 (TMOD & TMOD0_TOGGLE)  ?
 "1(toggle mode)"	:
 "0(interval mode)"
 );

// [2] Timer 0 initial TOUT0 value (TCLR0) /////////////////////////////////////
 Print("\nTimer 0 initial TOUT0 value.%s",
  (TMOD & TMOD0_TOUT) 	?
  "1"			:
  "0"
 ); 
 Print("\nTDATA0......................0x%08x",TDATA0);
 Print("\nTCNT0.......................0x%08x",TCNT0 );

// 
 Print("\nTimer0 TOUT0 output.(P16)...%s"	,
  ((IOPCON & IOPCON_TOEN0_EN)  && 	// T0UT0 enabled 
   (IOPMOD & 0x10000)) 	?		// P16 output
   "enabled"		:
   "disabled"
  );


 Print("\nTimer0 INT mode.............%s"	,
  (INTMODE & TIMER0_INT)?	
   "FIQ"		:		// FIQ mode 
   "IRQ"
  );

 Print("\nTimer0 interrupt enabled....%s"	,
  (INTMASK & TIMER0_INT)?
   "(disabled)"	:
   "(enabled)"
  );

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
 Print("\nTIMER 1 STATUS");

// TMOD1_EN // Timer 1 enable //////////////////////////////////////////////////
 Print("\nTimer 1 enable..............%s", 
  (TMOD & TMOD1_EN) 	?   
  "1(enabled."		:
  "0(disabled)"
  );
 
// Timer1 Toggle/Interval mode /////////////////////////////////////////////////
 Print("\nTimer 1 Toggle mode.........%s",
  (TMOD & TMOD1_TOGGLE) ?
  "1(toggle mode)"	:
  "0(interval mode)"
 );

// Timer 1 Initial TOUT1 value /////////////////////////////////////////////////
 Print("\nTimer 1 Initial TOUT1 value.%s",
  (TMOD & TMOD1_TOUT)	?
  "1"			:
  "0"
 );

//
 Print("\nTDATA1......................0x%08x",TDATA1);
 Print("\nTCNT1.......................0x%08x",TCNT1);

//
 Print("\nTimer1 TOUT1 output.(P17)...%s",
  ((IOPCON & IOPCON_TOEN1_EN) && 		// TOUT1 enabled 
   (IOPMOD & 0x20000))		?		// p17 output
   "1(enabled)"	:
   "0(disabled)"
  );

//
 Print("\nTimer1 INT mode.............%s"	,
  (INTMODE & TIMER1_INT)	?
   "FIQ"	:
   "IRQ"
  );

//
 Print("\nTimer1 interrupt enabled....%s"	,
  (INTMASK & TIMER1_INT) 	?
   "(disabled)"	:
   "(enabled)"
  );
}

/*************************************************************************/
/*                                                                       */
/* NAME : rtc_run()                                                      */
/*                                                                       */
/* FUNCTIONS : timer0 or 1 run test function.                            */
/*                                                                       */
/* EXAMPLES :                                                            */
/*                                                                       */
/* VARIABLES USED                                                        */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      06-07-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

static void rtc_run(void)
{
 U8 dev;  			/* for to select timer device 0,1 */
 U8 ch;   			/* variable for io consol */

// Select timer device. Timer0 or Timer 1 //////////////////////////////////////

 do 
  {
   Print("\nSelect timer[0/1]");
   dev = get_upper();
  } 
 while(dev != '1' && dev != '0');


// Now Start timer for test ////////////////////////////////////////////////////

 Print("\nTo exit, enter the Esc key on keyboard.");
 Print("\nTo start timer press any key.\n");
 ch = get_upper();
 if(ch == KEY_ESC)
  {
   return;
  }

 if(dev == '1') 
  {
   TimerInit(1,(ONE_SECOND/TICKS_PER_SECOND));
   TimerStart(1);
  }
 else 
  {
   TimerInit(0,(ONE_SECOND/TICKS_PER_SECOND));
   TimerStart(0);
  }

 while(1) 
  {
   char * s;
   TIME tm; 

   if(dev == '1') 
    { 
/* selected TIMER1 */ 
     GetSysTime(1, &tm);		/* get system time from TIMER1 */
     s = "TIMER1 time";			/* output string for TIMER1 */ 
    }
   else 
    {
/* selected TIMER0 */
     GetSysTime(0, &tm);		/* get system time from TIMER0 */
     s = "TIMER0 time";			/* output string for TIMER0 */
    }

   Print
    (
     "\n%s is %d:%d:%d.%d",
       s		,
      tm.tm_hour	,
      tm.tm_min		,
      tm.tm_sec		,
      tm.tm_ms
    );

   if(kbd_hit())
    {
     ch = get_byte();
     if(ch == KEY_ESC)
      {
       break;
      } 
    }
  } 

 if(dev == '1') 
  { 
   TimerReset(1);
  }
 else
  { 
   TimerReset(0);
  }

// Disable_Int(nGLOBAL_INT);    		/* disable interrupt source */
}


/*************************************************************************/
/*                                                                       */
/* NAME : TimerInit(timer device, time periode)                          */
/*                                                                       */
/* FUNCTIONS : Initialize the TIMER0,1.                                  */
/*                                                                       */
/* EXAMPLES :                                                            */
/*             TimerInit(TIMER_DEV0,ONE_SECOND/TICKS_PER_SECOND);          */
/*                                                                       */
/*               Where, the TIMER_DEV0 means timer0.                     */
/*                          ONE_SECOND = 1000,                           */
/*                          TICKS_PER_SECOND = 100,                      */
/*                then timer0 timer periode become to 10ms.              */
/*                                                                       */
/* VARIABLES USED                                                        */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      06-07-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

void TimerInit(int num, int ms) 
{
 if(num == 1)  
  {
/* for TIMER 1 */
   Disable_Int(nTIMER1_INT);    
   SysSetInterrupt(nTIMER1_INT, tm1isr);

   TDATA1 = t_data_ms(ms);    		/* unit is [ms] */
   TCNT1  = 0x0;
 
#ifdef TIMER_TOUT
   TMOD  |= TMOD1_TOGGLE; 	      	/* Toggle pulse will be out to port */
#else
   TMOD  &= ~TMOD1_TOGGLE;       	/* Interval mode */
#endif
   Enable_Int(nTIMER1_INT);  		/* Timer interrupt enable */
   Enable_Int(nGLOBAL_INT);		/* enable global interrupt */
  }
 else  
 if(num == 0)
  {
/* for TIMER0 */
   Disable_Int(nTIMER0_INT);    	/* disable interrupt source */
   SysSetInterrupt(nTIMER0_INT, tm0isr);/* set timer ISR */ 

   TDATA0 = t_data_ms(ms);  		/* timer period */
   TCNT0  = 0x0;			/* timer 0 counter */
#ifdef TIMER_TOUT
   TMOD  |= TMOD0_TOGGLE;        
#else
   TMOD  &= ~TMOD0_TOGGLE;        	/* Interval mode */
#endif
   Enable_Int(nTIMER0_INT); 		/* Enable interrupt source */	
   Enable_Int(nGLOBAL_INT);		/* enable global interrupt */
  }
}

// return 1 if timer num enabled, else return 0

int TimerEnabled(int num) 
{
/* Timer 0 enable */
 if( 
    ((num == 0) && (TMOD & TMOD0_EN) != 0 && (INTMASK & TIMER0_INT) == 0) ||
    ((num == 1) && (TMOD & TMOD1_EN) != 0 && (INTMASK & TIMER1_INT) == 0)
  )
  {					/* timer enabled, int enabled */
   return(1);
  } 
 else
  {
   return(0);
  }  
}


/*************************************************************************/
/*                                                                       */
/* NAME : tmReset(timer device )                                         */
/*                                                                       */
/* FUNCTIONS : Clear and initialize the TIMER0 or TIMER1                 */
/*                                                                       */
/* EXAMPLES :                                                            */
/*             tmReset(TIMER_DEV0);                                      */
/*                                                                       */
/*               Where, the TIMER_DEV0 means timer0.                     */
/*                                                                       */
/* VARIABLES USED                                                        */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      06-07-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/


void TimerReset(int num)
{
 if(num == 1) 
 {
  TMOD &= ~TMOD1_EN;       	/* Clear Timer mode register */     
  TDATA1 = 0;             	/* Clear Timer data register */
  TCNT1  = 0xffffffff;    	/* Clear timer counter register */

/* clear timer 1 variables */
  tm1.tm_ms   = 0;   		/* millisecond */
  tm1.tm_sec  = 0;   		/* second */
  tm1.tm_min  = 0;		/* minute */
  tm1.tm_hour = 0;		/* hour */
  tm1.tm_mday = 0;		/* month day */	
  tm1.tm_mon  = 0;		/* month */
  tm1.tm_year = 0;		/* year */
//
  clk_tick1   = 0;		/* timer tick */
 }
else 
if(num == 0)
 {
  TMOD  &= ~TMOD0_EN;       	/* Clear Timer mode register */     
  TDATA0 = 0;             	/* Clear Timer data register */
  TCNT0  = 0xffffffff;    	/* Clear timer counter register */

/* clear timer 0 variables */
  tm0.tm_ms   = 0;   		/* millisecond */	
  tm0.tm_sec  = 0;   		/* second */	
  tm0.tm_min  = 0;		/* minute */
  tm0.tm_hour = 0;		/* hour */
  tm0.tm_mday = 0;		/* month day */
  tm0.tm_mon  = 0;		/* month */
  tm0.tm_year = 0;		/* year */

//
  clk_tick0    = 0;		/* timer tick */
 }
}


/*************************************************************************/
/*                                                                       */
/* NAME : PrtSysTime(timer device, timer display string)                 */
/*                                                                       */
/* FUNCTIONS : Hour:Minutes:second of the timer device will be displyed  */
/*                                                                       */
/* EXAMPLES :                                                            */
/*             PrtSysTime(TIMER_DEV0,"Total test time");                 */
/*                                                                       */
/*             Print: Total test time is 3:30:20.8                       */
/*                                                                       */
/* VARIABLES USED                                                        */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      06-07-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

void GetSysTime(int num, TIME * tm)
{
 if(num == 1) 
  { 
/* get time from timer 1 */
   tm->tm_ms   = (clk_tick1%TICKS_PER_SECOND);  /* millisecond */
   tm->tm_sec  = tm1.tm_sec;			/* second */	
   tm->tm_sec += (int)(clk_tick1/TICKS_PER_SECOND); 
   tm->tm_min  = tm1.tm_min;			/* minute */
   tm->tm_hour = tm1.tm_hour;			/* hour */
   tm->tm_mday = tm1.tm_mday;			/* month day */	
   tm->tm_mon  = tm1.tm_mon;			/* month */
   tm->tm_year = tm1.tm_year;			/* year */
  }
 else
 if(num == 0)
  {
/* get time from timer 0 */
   tm->tm_ms   = (clk_tick0%TICKS_PER_SECOND); 	/* millisecond */
   tm->tm_sec  = tm0.tm_sec;			/* second */
   tm->tm_sec += (int)(clk_tick0/TICKS_PER_SECOND); 
   tm->tm_min  = tm0.tm_min;			/* minute */
   tm->tm_hour = tm0.tm_hour;			/* hour */
   tm->tm_mday = tm0.tm_mday;			/* month day */	
   tm->tm_mon  = tm0.tm_mon;			/* month */
   tm->tm_year = tm0.tm_year;			/* year */
  }
}


int TimerWaitMs(int tn, U32 mswait)
{
 int enabled;
 U32 msstart;					// start millisecond 
 U32 mscur;					// current millisecond
 TIME tm;					// TIME struct 

 if(tn < 0 || tn > 1) {			// check timer num
   return(0);
 } 
 
 enabled = TimerEnabled(tn); 		
 if(enabled == 0) {				// timer num not started => start 	
   TimerReset(tn);				// reset timer 0
   TimerInit(tn,(ONE_SECOND/TICKS_PER_SECOND));// init timer  0
   TimerStart(tn);				// start timer 0
 }

 Enable_Int(nGLOBAL_INT);			/* enable global interrupt */
 GetSysTime(tn, & tm);	
 mscur = msstart = tm.tm_ms + (tm.tm_sec + tm.tm_min * 60) * 1000;

 while(mscur < msstart + mswait) {
   GetSysTime(tn, & tm);
   mscur = tm.tm_ms + (tm.tm_sec + tm.tm_min * 60) * 1000;
 }

 if(enabled == 0) {				// restore timer state 
   TimerStop(tn);
 }
 return(1);
} 


/*************************************************************************/
/*                                                                       */
/* NAME : tm0isr()                                                       */
/*                                                                       */
/* FUNCTIONS : Timer0 interrupt service routine.                         */
/*                                                                       */
/* EXAMPLES :                                                            */
/*                                                                       */
/*       SysSetInterrupt(nTIMER0_INT, tm0isr);                           */
/*                                                                       */
/* VARIABLES USED                                                        */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      06-07-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

void tm0isr(void)
{
 clk_tick0++;  

 if(clk_tick0 >= TICKS_PER_SECOND) 
  {
   clk_tick0 = 0;
   if(tm0.tm_sec++ > 59) 
    { 
     tm0.tm_sec = 0;
     if(tm0.tm_min++ > 59) 
      {  
       tm0.tm_min = 0;
       if(tm0.tm_hour++ > 23) 
        {
         tm0.tm_hour = 0;
         if(tm0.tm_mday++ > 30) 
          {
           tm0.tm_mday = 0;
           if(tm0.tm_mon++ > 11) 
            { 
             tm0.tm_mon = 0; 
             tm0.tm_year++;
            }
          }
        }
      }
    }
  }  
}



/*************************************************************************/
/*                                                                       */
/* NAME : tm1isr()                                                       */
/*                                                                       */
/* FUNCTIONS : Timer1 interrupt service routine.                         */
/*                                                                       */
/* EXAMPLES :                                                            */
/*                                                                       */
/*       SysSetInterrupt(nTIMER1_INT, tm1isr);                           */
/*                                                                       */
/* VARIABLES USED                                                        */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      06-07-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

void tm1isr(void)
{
 clk_tick1++;  

 if(clk_tick1 >= TICKS_PER_SECOND) 
  {
   clk_tick1 = 0;
   if(tm1.tm_sec++ > 59) 
    { 
     tm1.tm_sec = 0;
     if(tm1.tm_min++ > 59) 
      {  
       tm1.tm_min = 0;
       if(tm1.tm_hour++ > 23) 
        {
         tm1.tm_hour = 0;
         if(tm1.tm_mday++ > 30) 
          {
           tm1.tm_mday = 0;
           if(tm1.tm_mon++ > 11) 
            { 
             tm1.tm_mon = 0; 
             tm1.tm_year++;
            }
          }
        }
      }
    }
  }  
}

/*************************************************************************/
/*                                                                       */
/* NAME : WatchDog1Lisr()                                                */
/*                                                                       */
/* FUNCTIONS : This function will be served when the watchdog timer      */
/*             count value is counted out.                               */
/*             If TOUT1(P17) merged to System reset input(nRESET),       */
/*             The system will be reset by the input signal from TOUT1.  */
/*             Any other case, rebooted by boot ROM.                     */
/*                                                                       */
/* EXAMPLES :                                                            */
/*                                                                       */
/*                                                                       */
/* VARIABLES USED                                                        */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      06-07-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

#define WDT_HW_RESET

static void WatchDog1Lisr(void)
{
#ifdef WDT_HW_RESET
 Print("\n\nSystem hardware reset.\n");
 Print("Reset signal came from TOUT1 as reset input\n");

/* This for the System hardware reset */
/* P17 have been merged system reset input pin */

 IOPDATA &= ~(1<<17);  				/* Output 0 to P17(TOUT1) */
 IOPMOD  |=  (1<<17);    			/* P17 output mode enable */

#else
 Print("\nSystem software reset.\n");
 Print("System will be reboot by boot ROM.\n");
 return;
#endif
     
 WatchDogStatus = 1;
}


/*************************************************************************/
/*                                                                       */
/* NAME : WatchDog0Lisr()                                                */
/*                                                                       */
/* FUNCTIONS : Basic system timer using timer0.                          */
/*                                                                       */
/* EXAMPLES :                                                            */
/*                                                                       */
/*                                                                       */
/* VARIABLES USED                                                        */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      06-07-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

static void WatchDog0Lisr(void)
{
 clk_tick0++;  

 if(clk_tick0 == TICKS_PER_SECOND) 
  {
   clk_tick0 = 0;
   if(tm0.tm_sec++ > 59) 
    { 
     tm0.tm_sec = 0;
     if(tm0.tm_min++ > 59) 
      {  
       tm0.tm_min = 0;
       if(tm0.tm_hour++ > 23) 
        {
         tm0.tm_hour = 0;
         if(tm0.tm_mday++ > 30) 
          {
           tm0.tm_mday = 0;
           if(tm0.tm_mon++ > 11) 
            { 
             tm0.tm_mon = 0; 
             tm0.tm_year++;
            }
          }
        }
      }
    }
  }  

 Disable_Int(nGLOBAL_INT);  		
 Disable_Int(nTIMER1_INT);    		/* Disable TIMER1 interrupt source */
 TimerStop(1);				/* Stop TIMER 1 */ 
 TDATA1 = t_data_ms(3*ONE_SECOND/TICKS_PER_SECOND); 	/* Set watchdog time to 3sec */
 WatchDogStatus = 0;			/* Clear WatchDog Status */
 Enable_Int(nTIMER1_INT);  		/* Timer1 interrupt source enable */
 Enable_Int(nGLOBAL_INT);  			/* Timer1 interrupt enable */
 TimerStart(1);				/* start timer 1 */
}

/*************************************************************************/
/*                                                                       */
/* NAME : WatchDogFunc()                                                 */
/*                                                                       */
/* FUNCTIONS : Watchdog timer functions.                                 */
/*             TIMER0 used to basic system timer.                        */
/*             TIMER1 used to watchdog timer.                            */
/*                                                                       */
/*             Whenever the system timer0 interrupt occurred, the watch  */
/*             dog timer(timer1) count value will be reloaded at timer0  */
/*             interrupt service routine as call WatchDogReload().       */
/*                                                                       */
/* EXAMPLES :                                                            */
/*                                                                       */
/*                                                                       */
/* VARIABLES USED                                                        */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      06-07-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

static void WatchDogInit(int reconf)
{
 Disable_Int(nGLOBAL_INT);  			
 Disable_Int(nTIMER0_INT);    			/* Disable TIMER0 interrupt */ 
 Disable_Int(nTIMER1_INT);    			/* Disable TIMER1 interrupt */

 if(reconf) 					/* timer0 reconfiguration */
  {
/* restore TIMER 0 ISR, without WatchDog reload	*/ 
   SysSetInterrupt(nTIMER0_INT, tm0isr);
  }
 else 
  {
   SysSetInterrupt(nTIMER0_INT, WatchDog0Lisr);
  } 
 
 SysSetInterrupt(nTIMER1_INT, WatchDog1Lisr);

/* TIMER 0 used to system basic timer */
 TDATA0 = t_data_ms(ONE_SECOND/TICKS_PER_SECOND); /* timer 0 period 10 ms */ 	  
 TCNT0 = 0x0;					/* timer0 counter */ 
 TMOD  = 0;        				/* Timer 0/1 mode register */


/* TIMER 1 used to system watchdog timer */


/* Watchdog timer start */

/* WARNING ! Check 74HCT125, pin 4 active 0 ? */
 IOPDATA |= (1<<17);  		       		/* Output 1 to P17(TOUT1) */
 IOPMOD  |= (1<<17);                		/* Enable P17 to output port */ 
                                           	/* TOUT1 will be cleared */
 TCNT1  = 0;                          		/* Clear timer counter register */
 TDATA1 = t_data_ms(3*ONE_SECOND/TICKS_PER_SECOND); /* Set watchdog time to 3sec */
                                   		/* Interval mode,timer start */
 WatchDogStatus = 0;

/* Enable TIMER0/1 interrupts */  
 Enable_Int(nTIMER0_INT);  			/* Timer0 interrupt enable */
 Enable_Int(nTIMER1_INT);  			/* Timer1 interrupt enable */
 Enable_Int(nGLOBAL_INT);  			/* Timer1 interrupt enable */

/* TIMER RUN enable */
 TimerStart(0);			/* Start TIMER 0 */
 TimerStart(1);			/* Start TIMER 1 */
}


/*************************************************************************/
/*                                                                       */
/* NAME : WatchDogTest()                                                 */
/*                                                                       */
/* FUNCTIONS : Watchdog timer function test.                             */
/*                                                                       */
/* EXAMPLES :                                                            */
/*                                                                       */
/* VARIABLES USED                                                        */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      06-07-1999      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

static void WatchDogTest(void)
{
 U8 ch;

 Print("\n\nWatchDog Timer function test.");
 Print("\nTimer0 used as system timer.");
 Print("\nTimer1 used as watchdog timer.");
 Print("\nWatchdog timer counter value will be initialized at");   
 Print("\nsystem timer interrupt service routine repeatedly.");   
 Print("\nIf you want to make the system hangup situation,");
 Print("\nEnter the character 'R'.\n");

 WatchDogInit(0);
 Print("\n\n Now, watchdog & system timer is running.....\n");

 while(1) 
  {
   ch = get_upper(); 			/* get byte from console */

   if(ch == 'R')
    {
     Print("\nTIMER0 reconfigured with no watchdog Init function.\n");

/* TIMER 0 re-configured with no watchdog Initialize function */

     WatchDogInit(1);
     Print("\nNow, wathdog timer interrupt occurred...\n");
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
// timer test top function /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void TimerTest(void)
{
 U8 it;
 while(1) 
  {
   Print("\n\nTIMER TEST"); 
   Print("\n[R] Running Timer 0,1."); 
   Print("\n[W] Watchdog Timer function(Timer1) Test.");
   Print("\n[V] View the timer configurations.");
   Print("\n[Q] Type Q for exit timer test.");
   Print("\nSelect Test Item: ");
   it = get_upper();

   switch(it) 
    {
     case 'R' : rtc_run();  break;	  	/* TIMER0/TIMER1 test */
     case 'W' : WatchDogTest();  break;   	/* WatchDog Test */	
     case 'V' : TimerConfigView();  break; 	/* show timer0/timer1 configuration */
     case 'Q' : return;
     default  : break;
    }

   Print("\nPress any key to continue.");
   it = get_upper();
   if(it == 'Q')
    {
     break;
    } 
  } 
}



